/*
 * $Id: $
 * 
 * $Copyright: (c) 2018 Broadcom.
 * Broadcom Proprietary and Confidential. All rights reserved.$
 * 
 * DO NOT EDIT THIS FILE!
 * This file is auto-generated.
 * Edits to this file will be lost when it is regenerated.
 */

#ifndef __BCM_UDF_H__
#define __BCM_UDF_H__

#include <bcm/types.h>
#include <bcm/field.h>

/* Options to control UDF allocation */
#define BCM_UDF_CREATE_O_NONE           (0)        /* Default. */
#define BCM_UDF_CREATE_O_WITHID         (1 << 0)   /* Use input UDF ID instead
                                                      of allocating one. */
#define BCM_UDF_CREATE_O_REPLACE        (1 << 1)   /* Replace/Update existing
                                                      UDF object. */
#define BCM_UDF_CREATE_O_FLEXHASH       (1 << 2)   /* UDF allocation will be
                                                      adjusted for use by flex
                                                      hashing feature. */
#define BCM_UDF_CREATE_O_FIELD_INGRESS  (1 << 3)   /* UDF allocation will be
                                                      adjusted for use by
                                                      Ingress stage FP. */
#define BCM_UDF_CREATE_O_FIELD_LOOKUP   (1 << 4)   /* UDF allocation will be
                                                      adjusted for use by Lookup
                                                      stage FP. */
#define BCM_UDF_CREATE_O_SHARED_HWID    (1 << 5)   /* Use same hardware
                                                      resources as 'shared_udf' */
#define BCM_UDF_CREATE_O_POLICER_GROUP  (1 << 6)   /* UDF allocation will be
                                                      adjusted for use by global
                                                      (service) meter. */
#define BCM_UDF_CREATE_O_UDFHASH        (1 << 7)   /* UDF allocation will be
                                                      adjusted for use by udf
                                                      hashing feature. */
#define BCM_UDF_CREATE_O_RANGE_CHECK    (1 << 8)   /* UDF allocation will be
                                                      adjusted for use by Range
                                                      checker. */
#define BCM_UDF_CREATE_O_FLEX_COUNTER   (1 << 9)   /* UDF allocation will be
                                                      adjusted for use by
                                                      Flexible Counter. */

/* UDF Packet Format Options */
#define BCM_UDF_PKT_FORMAT_CREATE_O_NONE    0          /* Default. */
#define BCM_UDF_PKT_FORMAT_CREATE_O_WITHID  (1 << 0)   /* Create packet format
                                                          with given ID */
#define BCM_UDF_PKT_FORMAT_CREATE_O_REPLACE (1 << 1)   /* Update existing packet
                                                          format */

/* Packet format options */
typedef uint32 bcm_udf_pkt_format_options_t;

/* Flags to be used with bcm_udf_t.flags */
#define BCM_UDF_F_ADJUST_IP4_OPTIONS    (1 << 1)   /* Adjust offset accounting
                                                      IP4 options */
#define BCM_UDF_F_ADJUST_IP6_OPTIONS    (1 << 2)   /* Adjust offset accounting
                                                      IP6 Extension Headers */

/* Base header the switch can parse. */
typedef enum bcm_udf_layer_e {
    bcmUdfLayerL2Header = 1,        /* Start of L2 header in the Packet */
    bcmUdfLayerL3OuterHeader = 2,   /* Start of outer L3 header in the packet */
    bcmUdfLayerL3InnerHeader = 3,   /* Start of inner L3 header in the packet */
    bcmUdfLayerL4OuterHeader = 4,   /* Start of outer L4 header in the packet */
    bcmUdfLayerL4InnerHeader = 5,   /* Start of outer L4 header in the packet */
    bcmUdfLayerHigigHeader = 6,     /* Start of Higig header in the packet */
    bcmUdfLayerHigig2Header = 7,    /* Start of Higig2 header in the packet */
    bcmUdfLayerTunnelHeader = 8,    /* Start of outermost tunnel header in the
                                       Packet */
    bcmUdfLayerTunnelPayload = 9,   /* Start of outermost tunnel payload in the
                                       Packet */
    bcmUdfLayerUserPayload = 10,    /* Start of application payload in the
                                       Packet. */
    bcmUdfLayerCount                /* Unused. Always last in the list */
} bcm_udf_layer_t;

#define BCM_UDF_LAYER \
{ \
    "L2Header", \
    "L3OuterHeader", \
    "L3InnerHeader", \
    "L4OuterHeader", \
    "L4InnerHeader", \
    "HigigHeader", \
    "Higig2Header", \
    "TunnelHeader", \
    "TunnelPayload", \
    "UserPayload"  \
}

/* UDF structure. */
typedef struct bcm_udf_s {
    uint32 flags;           /* See BCM_UDF_F_XXX */
    bcm_udf_layer_t layer;  /* UDF base offset. bcmUdfLayerXxx */
    int start;              /* Relative offset in bits (from the 'layer') in the
                               packet */
    uint32 width;           /* width of data in bits to extract */
    bcm_pbmp_t ports;       /* ports (must belong to a single pipe) associate
                               with the udf_offset: used only in PipeLocal
                               operation supported devices */
} bcm_udf_t;

/* UDF allocation hints. */
typedef struct bcm_udf_alloc_hints_s {
    uint32 flags;               /* See BCM_UDF_CREATE_O_xxx */
    bcm_udf_id_t shared_udf;    /* Used when BCM_UDF_CREATE_O_SHARED_HWID is set */
    bcm_field_qset_t qset;      /* Field group qset for optimal H/W resource
                                   allocation */
} bcm_udf_alloc_hints_t;

/* Packet format based UDF specification structure. */
typedef struct bcm_udf_pkt_format_info_s {
    int prio;                           /* Priority of the UDF */
    bcm_ethertype_t ethertype;          /* Ethertype in the packet */
    bcm_ethertype_t ethertype_mask;     /* Ethertype Mask */
    uint8 ip_protocol;                  /* IP protocol field in the packet */
    uint8 ip_protocol_mask;             /* IP protocol Mask */
    uint16 l2;                          /* L2 packet format.
                                           (BCM_PKT_FORMAT_L2_XXX) */
    uint16 vlan_tag;                    /* Vlan tag format.
                                           (BCM_PKT_FORMAT_VLAN_XXX) */
    uint16 outer_ip;                    /* Outer IP header type.
                                           (BCM_PKT_FORMAT_IP_XXX) */
    uint16 inner_ip;                    /* Inner IP header type.
                                           (BCM_PKT_FORMAT_IP_XXX) */
    uint16 tunnel;                      /* Tunnel type.
                                           (BCM_PKT_FORMAT_TUNNEL_XXX) */
    uint16 mpls;                        /* Mpls labels.
                                           (BCM_PKT_FORMAT_MPLS_XXX) */
    uint16 fibre_chan_outer;            /* Fibre Channel outer header type.
                                           (BCM_PKT_FORMAT_FIBRE_CHAN_XXX) */
    uint16 fibre_chan_inner;            /* Fibre Channel inner header
                                           type.(BCM_PKT_FORMAT_FIBRE_CHAN_XXX) */
    uint16 higig;                       /* HIGIG packet
                                           format.(BCM_PKT_FORMAT_HIGIG_XXX) */
    uint16 vntag;                       /* NIV packet
                                           format.(BCM_PKT_FORMAT_VNTAG_XXX) */
    uint16 etag;                        /* Extender tag packet
                                           format.(BCM_PKT_FORMAT_ETAG_XXX) */
    uint16 cntag;                       /* CNTAG packet
                                           format.(BCM_PKT_FORMAT_CNTAG_XXX) */
    uint16 icnm;                        /* ICNM packet
                                           format.(BCM_PKT_FORMAT_ICNM_XXX) */
    uint16 subport_tag;                 /* SUBPORT_TAG packet format.
                                           (BCM_PKT_FORMAT_SUBPORT_TAG_XXX) */
    int class_id;                       /* id used by the IFP Key Selection
                                           Mechanism. Range 0~255 */
    uint16 inner_protocol;              /* Inner protocol field in the packet.
                                           Inner_ip should be set to
                                           BCM_PKT_FORMAT_IP_NONE to make this
                                           take effect. IP-IN-IP type GRE should
                                           take original API sequence. */
    uint16 inner_protocol_mask;         /* Inner protocol mask. */
    uint32 l4_dst_port;                 /* TCP or UDP Destination Port number. */
    uint32 l4_dst_port_mask;            /* TCP or UDP Destination Port number
                                           mask. */
    uint8 opaque_tag_type;              /* Tag type field in the opaque tag. */
    uint8 opaque_tag_type_mask;         /* Mask for tag type field in the opaque
                                           tag. */
    uint8 int_pkt;                      /* Inband Telemetry packet indicator. */
    uint8 src_port;                     /* Logical Source Port number. */
    uint8 src_port_mask;                /* Logical Source Port number mask. */
    uint8 lb_pkt_type;                  /* Loopback Packet Type.
                                           (BCM_PKT_FORMAT_LB_TYPE_XXX) */
    uint16 first_2bytes_after_mpls_bos; /* First 2 bytes after BOS in an mpls
                                           packet. */
    uint16 first_2bytes_after_mpls_bos_mask; /* Mask for first 2 bytes after BOS in
                                           an mpls packet. */
} bcm_udf_pkt_format_info_t;

/* Maxmium bytes of UDF Hash */
#define BCM_UDF_HASH_MAX_LENGTH 4          

/* UDF hashing configuration. */
typedef struct bcm_udf_hash_config_s {
    uint32 flags;                       /* Reserved for future usage. Must be
                                           zero today. */
    bcm_udf_id_t udf_id;                /* UDF object id */
    int mask_length;                    /* Length of mask bytes. Must equal to
                                           the width of the UDF object. */
    uint8 hash_mask[BCM_UDF_HASH_MAX_LENGTH]; /* Array of mask fields. Index 0 is MSB
                                           of mask. */
} bcm_udf_hash_config_t;

/* UDF hashing engine selection. */
#define BCM_UDF_HASH_CONFIG_ENGINE_0    0x00000001 /* UDF is used by hash engine
                                                      0. */
#define BCM_UDF_HASH_CONFIG_ENGINE_1    0x00000002 /* UDF is used by hash engine
                                                      1. */

/* Options to control UDF hashing adding. */
#define BCM_UDF_HASH_ADD_O_NONE     0          /* Default, add new entry to
                                                  list. */
#define BCM_UDF_HASH_ADD_O_REPLACE  (1 << 0)   /* Modify the entry which is
                                                  already in list */

/* Initializes the UDF structure */
extern void bcm_udf_t_init(
    bcm_udf_t *udf_info);

/* Initializes the UDF alloc hints structure */
extern void bcm_udf_alloc_hints_t_init(
    bcm_udf_alloc_hints_t *udf_hints);

/* Initialize the UDF packet format structure */
extern void bcm_udf_pkt_format_info_t_init(
    bcm_udf_pkt_format_info_t *pkt_format);

/* Initialize UDF hashing configuration structure. */
extern void bcm_udf_hash_config_t_init(
    bcm_udf_hash_config_t *info);

#ifndef BCM_HIDE_DISPATCHABLE

/* Initialize UDF module */
extern int bcm_udf_init(
    int unit);

/* Detach UDF module */
extern int bcm_udf_detach(
    int unit);

/* Creates a UDF object */
extern int bcm_udf_create(
    int unit, 
    bcm_udf_alloc_hints_t *hints, 
    bcm_udf_t *udf_info, 
    bcm_udf_id_t *udf_id);

/* Fetches the UDF object created in the system */
extern int bcm_udf_get(
    int unit, 
    bcm_udf_id_t udf_id, 
    bcm_udf_t *udf_info);

/* Fetches all existing UDF ids */
extern int bcm_udf_get_all(
    int unit, 
    int max, 
    bcm_udf_id_t *udf_id_list, 
    int *actual);

/* Destroys the UDF object */
extern int bcm_udf_destroy(
    int unit, 
    bcm_udf_id_t udf_id);

/* Create a packet format entry */
extern int bcm_udf_pkt_format_create(
    int unit, 
    bcm_udf_pkt_format_options_t options, 
    bcm_udf_pkt_format_info_t *pkt_format, 
    bcm_udf_pkt_format_id_t *pkt_format_id);

/* Retrieve packet format info given the packet format Id */
extern int bcm_udf_pkt_format_info_get(
    int unit, 
    bcm_udf_pkt_format_id_t pkt_format_id, 
    bcm_udf_pkt_format_info_t *pkt_format);

/* Destroy existing packet format entry */
extern int bcm_udf_pkt_format_destroy(
    int unit, 
    bcm_udf_pkt_format_id_t pkt_format_id);

/* Adds packet format entry to UDF object */
extern int bcm_udf_pkt_format_add(
    int unit, 
    bcm_udf_id_t udf_id, 
    bcm_udf_pkt_format_id_t pkt_format_id);

/* Deletes packet format spec associated with the UDF */
extern int bcm_udf_pkt_format_get(
    int unit, 
    bcm_udf_pkt_format_id_t pkt_format_id, 
    int max, 
    bcm_udf_id_t *udf_id_list, 
    int *actual);

/* Deletes packet format spec associated with the UDF */
extern int bcm_udf_pkt_format_delete(
    int unit, 
    bcm_udf_id_t udf_id, 
    bcm_udf_pkt_format_id_t pkt_format_id);

/* Retrieves the user defined format specification configuration from UDF */
extern int bcm_udf_pkt_format_get_all(
    int unit, 
    bcm_udf_id_t udf_id, 
    int max, 
    bcm_udf_pkt_format_id_t *pkt_format_id_list, 
    int *actual);

/* Deletes all packet format specs associated with the UDF */
extern int bcm_udf_pkt_format_delete_all(
    int unit, 
    bcm_udf_id_t udf_id);

/* Add UDF id into hashing list and configure it. */
extern int bcm_udf_hash_config_add(
    int unit, 
    uint32 options, 
    bcm_udf_hash_config_t *config);

/* Delete UDF id from hashing list. */
extern int bcm_udf_hash_config_delete(
    int unit, 
    bcm_udf_hash_config_t *config);

/* Delete all UDF id from hashing list. */
extern int bcm_udf_hash_config_delete_all(
    int unit);

/* Get UDF hashing configuration of a certain id. */
extern int bcm_udf_hash_config_get(
    int unit, 
    bcm_udf_hash_config_t *config);

/* Get all added UDF ids from list. */
extern int bcm_udf_hash_config_get_all(
    int unit, 
    int max, 
    bcm_udf_hash_config_t *config_list, 
    int *actual);

#endif /* BCM_HIDE_DISPATCHABLE */

/* 
 * List of UDF operational modes supported: used only in PipeLocal
 * operation supported devices
 */
typedef enum bcm_udf_oper_mode_e {
    bcmUdfOperModeGlobal = 0,       /* UDF Operational Global Mode Value. */
    bcmUdfOperModePipeLocal = 1,    /* UDF Operational Pipe Local Mode Value. */
    bcmUdfOperModeCount = 2         /* Always last. Not a usable value. */
} bcm_udf_oper_mode_t;

#define BCM_UDF_OPER_MODE_STRINGS \
{ \
    "Global", \
    "PipeLocal"  \
}

#ifndef BCM_HIDE_DISPATCHABLE

/* 
 * Configure udf operation mode: used only in PipeLocal operation
 * supported devices
 */
extern int bcm_udf_oper_mode_set(
    int unit, 
    bcm_udf_oper_mode_t mode);

/* 
 * Retrieves current udf operation mode: used only in PipeLocal operation
 * supported devices
 */
extern int bcm_udf_oper_mode_get(
    int unit, 
    bcm_udf_oper_mode_t *mode);

#endif /* BCM_HIDE_DISPATCHABLE */

/* Base header the switch can parse. */
typedef enum bcm_udf_abstract_pkt_format_e {
    bcmUdfAbstractPktFormatLlc = 0,     /* Abstract from the start of first byte
                                           of LLC Header. */
    bcmUdfAbstractPktFormatInnerLlc = 1, /* Abstract from the start of Inner
                                           header's first byte of LLC Header. */
    bcmUdfAbstractPktFormatUnknownL3 = 2, /* Abstract from the start of first byte
                                           after unknown ethertype. */
    bcmUdfAbstractPktFormatInnerUnknownL3 = 3, /* Abstract from the start of inner
                                           header's first byte after unknown
                                           ethertype. */
    bcmUdfAbstractPktFormatKnownNonIp = 4, /* Abstract from the start of first byte
                                           after Known non-ip ethertype (other
                                           than Known IPv4/6 and FcoE/Mim/MPLS
                                           tunnels). */
    bcmUdfAbstractPktFormatInnerKnownNonIp = 5, /* Abstract from the start of Inner
                                           header's first byte after Known
                                           non-ip ethertype (other than Known
                                           IPv4/6 and FcoE/Mim/MPLS tunnels). */
    bcmUdfAbstractPktFormatKnownL3Mim = 6, /* Abstract from the start of MiM
                                           Header. */
    bcmUdfAbstractPktFormatKnownL3FcoeStdEncap = 7, /* Abstract from the start of FcoE with
                                           Standard or Encapsulation Header. */
    bcmUdfAbstractPktFormatKnownL3FcoeVftIfr = 8, /* Abstract from the start of FcoE with
                                           Virtual Fabric Tag (VFT) or Inter
                                           Fabric Routing (IFR) Header. */
    bcmUdfAbstractPktFormatKnownL3MplsOneLabel = 9, /* Abstract from the start of MPLS
                                           Header with one label. */
    bcmUdfAbstractPktFormatKnownL3MplsTwoLabel = 10, /* Abstract from the start of MPLS
                                           Header with two labels. */
    bcmUdfAbstractPktFormatKnownL3MplsThreeLabel = 11, /* Abstract from the start of MPLS
                                           Header with three labels. */
    bcmUdfAbstractPktFormatKnownL3MplsFourLabel = 12, /* Abstract from the start of MPLS
                                           Header with four labels. */
    bcmUdfAbstractPktFormatKnownL3MplsMoreThanFourLabel = 13, /* Abstract from the start of MPLS
                                           Header with more than four labels. */
    bcmUdfAbstractPktFormatUnknownL4 = 14, /* Abstract from the start of first byte
                                           of unknown L4 Header. */
    bcmUdfAbstractPktFormatInnerUnknownL4 = 15, /* Abstract from the start of Inner
                                           header's first byte of unknown L4
                                           Header. */
    bcmUdfAbstractPktFormatUdpUnknownL5 = 16, /* Abstract from the start of first byte
                                           of unknown L5 after UDP Header. */
    bcmUdfAbstractPktFormatInnerUdpUnknownL5 = 17, /* Abstract from the start of Inner
                                           header's first byte of unknown L5
                                           after UDP Header. */
    bcmUdfAbstractPktFormatUdpGpeGeneve = 18, /* Abstract from the start of first byte
                                           of known L5 Geneve header after UDP. */
    bcmUdfAbstractPktFormatUdpBfd = 19, /* Abstract from the start of first byte
                                           of known L5 BFD header after UDP. */
    bcmUdfAbstractPktFormatInnerUdpBfd = 20, /* Abstract from the start of Inner
                                           header's first byte of known L5 BFD
                                           header after UDP. */
    bcmUdfAbstractPktFormatUdpVxlan = 21, /* Abstract from the start of first byte
                                           of known L5 VXLAN header after UDP. */
    bcmUdfAbstractPktFormatUdp1588 = 22, /* Abstract from the start of first byte
                                           of known L5 1588 header after UDP. */
    bcmUdfAbstractPktFormatTcpUnknownL5 = 23, /* Abstract from the start of first byte
                                           of unknown L5 header after TCP. */
    bcmUdfAbstractPktFormatInnerTcpUnknownL5 = 24, /* Abstract from the start of Inner
                                           header's first byte of unknown L5
                                           header after TCP. */
    bcmUdfAbstractPktFormatSctpUnknownL5 = 25, /* Abstract from the start of first byte
                                           of unknown L5 header after SCTP. */
    bcmUdfAbstractPktFormatInnerSctpUnknownL5 = 26, /* Abstract from the start of Inner
                                           header's first byte of unknown L5
                                           header after SCTP. */
    bcmUdfAbstractPktFormatGreWithoutKey = 27, /* Abstract from the start of first byte
                                           of L4 header for GRE packets without
                                           key flag set (C=0/1, R=0/1 and K=0). */
    bcmUdfAbstractPktFormatGreWithKey = 28, /* Abstract from the start of first byte
                                           of L4 header for GRE packets with key
                                           flag set (C=1, R=0/1 and K=1). */
    bcmUdfAbstractPktFormatGreWithoutChecksumRouting = 29, /* Abstract from the start of first byte
                                           of L4 header for GRE packets (with
                                           flags C=0, R=0 and K=1). */
    bcmUdfAbstractPktFormatL2GreWithoutChecksumRouting = 30, /* Abstract from the start of first byte
                                           of L4 header for L2-GRE packets (with
                                           flags C=0, R=0, K=1). */
    bcmUdfAbstractPktFormatL2GreWithoutRouting = 31, /* Abstract from the start of first byte
                                           of L4 header for L2-GRE packets (with
                                           flags R=0, C=1, K=1). */
    bcmUdfAbstractPktFormatAchNonIp = 32, /* Abstract from the start of ACH Header
                                           for a NonIp packet. */
    bcmUdfAbstractPktFormatUdpINT = 33, /* Abstract from the start of first byte
                                           of UDP header to match on INT
                                           (Inband-Telemetry) header payload. */
    bcmUdfAbstractPktFormatTcpUnknownL5WithIpExtnHdr = 34, /* Abstract from the start of first byte
                                           of TCP header for unknown L5 packet. */
    bcmUdfAbstractPktFormatL3GreUnknownPayload = 35, /* Abstract from the start of first byte
                                           of L3 GRE header for unknown GRE type
                                           packets. */
    bcmUdfAbstractPktFormatGreWithKeyRouting = 36, /* Abstract from the start of first byte
                                           of L4 header for GRE packets (with
                                           key and routing flag set K=1, R=1,
                                           C=0/1). */
    bcmUdfAbstractPktFormatL2GreWithKeyRouting = 37, /* Abstract from the start of first byte
                                           of L4 header for L2-GRE packets (with
                                           flags K=1, R=1, C=0/1). */
    bcmUdfAbstractPktFormatInnerUdp1588 = 38, /* Abstract from the start of Inner
                                           header's first byte of known L5 1588
                                           header after UDP. */
    bcmUdfAbstractPktFormatTcpINT = 39, /* Abstract from the start of first byte
                                           of TCP header to match on INT
                                           (Inband-Telemetry) header payload. */
    bcmUdfAbstractPktFormatKnownL3FcoeUnknownRctl = 40, /* Abstract from the start of FcoE with
                                           Unknown R_CTL. */
    bcmUdfAbstractPktFormatLastCount = 41 /* Unused. Always last in the list */
} bcm_udf_abstract_pkt_format_t;

#define BCM_UDF_ABSTRACT_PKT_FORMAT \
{ \
    "Llc", \
    "InnerLlc", \
    "UnknownL3", \
    "InnerUnknownL3", \
    "KnownNonIp", \
    "InnerKnownNonIp", \
    "KnownL3Mim", \
    "KnownL3FcoeStdEncap", \
    "KnownL3FcoeVftIfr", \
    "KnownL3MplsOneLabel", \
    "KnownL3MplsTwoLabel", \
    "KnownL3MplsThreeLabel", \
    "KnownL3MplsFourLabel", \
    "KnownL3MplsMoreThanFourLabel", \
    "UnknownL4", \
    "InnerUnknownL4", \
    "UdpUnknownL5", \
    "InnerUdpUnknownL5", \
    "UdpGpeGeneve", \
    "UdpBfd", \
    "InnerUdpBfd", \
    "UdpVxlan", \
    "Udp1588", \
    "TcpUnknownL5", \
    "InnerTcpUnknownL5", \
    "SctpUnknownL5", \
    "InnerSctpUnknownL5", \
    "GreWithoutKey", \
    "GreWithKey", \
    "GreWithoutChecksumRouting", \
    "L2GreWithoutChecksumRouting", \
    "L2GreWithoutRouting", \
    "AchNonIp", \
    "UdpINT", \
    "TcpUnknownL5WithIpExtnHdr", \
    "L3GreUnknownPayload", \
    "GreWithKeyRouting", \
    "L2GreWithKeyRouting", \
    "InnerUdp1588", \
    "TcpINT", \
    "KnownL3FcoeUnknownRctl"  \
}

/* UDF Chunk Information Structure. */
typedef struct bcm_udf_chunk_info_s {
    int offset;                         /* Relative offset in bits (Starts from
                                           base offset of given abstract packet
                                           format) in the packet */
    uint32 width;                       /* width of data in bits to extract */
    uint32 chunk_bmap;                  /* Chunk bitmap (Each bit in the bitmap
                                           represents 1 chunk (2 bytes) */
    bcm_udf_abstract_pkt_format_t abstract_pkt_format; /* Abstract Packet Format (one of the
                                           format from
                                           'bcm_udf_abstract_pkt_format_t') */
} bcm_udf_chunk_info_t;

/* Initializes the UDF Chunk Info structure */
extern void bcm_udf_chunk_info_t_init(
    bcm_udf_chunk_info_t *udf_chunk_info);

/* Pkt Base offset. */
typedef enum bcm_udf_pkt_base_offset_e {
    bcmUdfPktBaseOffsetStartOfOuterL2 = 0, /* Base offset from the start of Outer
                                           L2 Header. */
    bcmUdfPktBaseOffsetStartOfOuterLlc = 1, /* Base offset from the start of Outer
                                           LLC Header. */
    bcmUdfPktBaseOffsetStartOfTunnel = 2, /* Base offset from the start of Tunnel
                                           Header. */
    bcmUdfPktBaseOffsetStartOfOuterL3 = 3, /* Base offset from the start of Outer
                                           L3 Header. */
    bcmUdfPktBaseOffsetStartOfOuterL4 = 4, /* Base offset from the start of Outer
                                           L4 Header. */
    bcmUdfPktBaseOffsetStartOfOuterL5 = 5, /* Base offset from the start of Outer
                                           L5 Header. */
    bcmUdfPktBaseOffsetStartOfInnerL2 = 6, /* Base offset from the start of Inner
                                           L2 Header. */
    bcmUdfPktBaseOffsetStartOfInnerLlc = 7, /* Base offset from the start of Inner
                                           LLC Header. */
    bcmUdfPktBaseOffsetStartOfInnerL3 = 8, /* Base offset from the start of Inner
                                           L3 Header. */
    bcmUdfPktBaseOffsetStartOfInnerL4 = 9, /* Base offset from the start of Inner
                                           L4 Header. */
    bcmUdfPktBaseOffsetStartOfInnerL5 = 10, /* Base offset from the start of Inner
                                           L5 Header. */
    bcmUdfPktBaseOffsetLastCount = 11   /* Unused. Always last in the list */
} bcm_udf_pkt_base_offset_t;

#define BCM_UDF_PKT_BASE_OFFSET \
{ \
    "StartOfOuterL2", \
    "StartOfOuterLlc", \
    "StartOfTunnel", \
    "StartOfOuterL3", \
    "StartOfOuterL4", \
    "StartOfOuterL5", \
    "StartOfInnerL2", \
    "StartOfInnerLlc", \
    "StartOfInnerL3", \
    "StartOfInnerL4", \
    "StartOfInnerL5"  \
}

/* UDF Abstract Packet Format Information Structure. */
typedef struct bcm_udf_abstract_pkt_format_info_s {
    bcm_udf_pkt_base_offset_t base_offset; /* Relative offset of
                                           'bcmUdfPktBaseOffsetXX' type (from
                                           'bcm_udf_pkt_base_offset_t') */
    uint32 num_chunks_max;              /* Maximum number of chunks supported by
                                           the abstract packet format */
    uint32 chunk_bmap_used;             /* Bitmap of Chunks already used by the
                                           abstract packet format */
} bcm_udf_abstract_pkt_format_info_t;

#ifndef BCM_HIDE_DISPATCHABLE

/* 
 * API to retrieve Abstract packet format information. To create a UDF
 * object for a given abstract packet format, user needs to know the
 * information about the start base offset in the packet header and
 * number of chunks supported and consumed to extract the UDF fields.
 */
extern int bcm_udf_abstract_pkt_format_info_get(
    int unit, 
    bcm_udf_abstract_pkt_format_t abstract_pkt_format, 
    bcm_udf_abstract_pkt_format_info_t *pkt_format_info);

/* Creates a UDF object */
extern int bcm_udf_chunk_create(
    int unit, 
    bcm_udf_alloc_hints_t *hints, 
    bcm_udf_chunk_info_t *udf_chunk_info, 
    bcm_udf_id_t *udf_id);

/* Get a UDF object information. */
extern int bcm_udf_chunk_info_get(
    int unit, 
    bcm_udf_id_t udf_id, 
    bcm_udf_chunk_info_t *udf_chunk_info);

/* 
 * Retrieve the list of UDF Objects associated with a given abstract
 * packet format
 */
extern int bcm_udf_abstract_pkt_format_object_list_get(
    int unit, 
    bcm_udf_abstract_pkt_format_t abstract_pkt_format, 
    int max, 
    bcm_udf_id_t *udf_id_list, 
    int *actual);

/* API to retrieve the chunk information supported for Range Checker. */
extern int bcm_udf_range_checker_chunk_info_get(
    int unit, 
    uint8 *num_chunks, 
    uint32 *chunk_bmap);

#endif /* BCM_HIDE_DISPATCHABLE */

#endif /* __BCM_UDF_H__ */
